# Assumes: IRAF ver 2.10.4, all images in list must be same size, must be
#    at least two images in the list
#
# BGSUB -- Given an list of input images,
# BGSUB will do background subtractions on that list by averaging
# the two images surrounding the input image and subtracting them from
# the image.  In the case where there are not two surrounding images
# (i.e., at the beginning and end of the list), BGSUB will simply subtract
# the nearest image.  The output consists of a list of images identical to
# the input list but with a '.r' appended.  The input list must contain at
# least two images.  BGSUB will also add a parameter to the image header
# called 'BGSUB' which lists the background that was subtracted from the
# image (or the two backgrounds which were averaged and subtracted from the
# image).  BGSUB prints out the image names of the new images as they are
# created, suitable for redirecting into a list to use as input for subsequent
# tasks.
#
# Example:  The file 'list' contains the image names m82.001 through m82.008.
# To do the background subtraction, we type:   bgsub @list m82 >m82list
# The result will be a series of background-subtracted images named m82.001.r
# through m82.008.r with new image names in the file m82list and with the new
# image headers updated with the 'BGSUB' keyword containing the image names
# that were subtracted from the new image.
#
# C.W. Engelbracht 9/27/93
#
# BUG FIX 9/27/93:
#    BGSUB previously crashed if the parameters of 'hedit' were not set the
#    way the task liked them to be.  BGSUB now sets the parameters of 'hedit'
#    explicitly.  (CWE)
#
# BUG FIX 10/13/94:
#    Added 'verify-' to imdelete command.  Added error checking to see if
#    images exist and to see if there is more than one image in the input list.
#    Also added checks to see if the .r images exist already.  Did a general
#    cleanup of error checking so that gobs of tmp* files are not left sitting
#    around.  Moved most of tmp files to tmp$.  (CWE)
#
# CHANGE 5/18/95:
#    Added the 'alternate' option (similar to the one in 'doall') which causes
#    the script to not subtract a background from even-numbered images. (CWE)
#
# CHANGE 4/1/96:
#    Added the 'median' background subtraction mode, in which the background
#    subtracted from all the images is the median of the input images.  (CWE)
#    Now you can specify the background-subtraction mode as:
#       norm	background is average of two surrounding images
#       alt	like norm, but only odd-numbered images are bg-subtracted
#       median     background is median of input list
#
# CHANGE 9/4/98:
#    Revised script to use .fits images rather than .imh.  (CWE)

procedure bgsub (images)

string   images   {prompt="list of input images"}
string   bgtype   {'norm', prompt = 'Background subtraction mode',
                     enum = 'norm|alt|median'}
struct   *flist
struct   *tlist

begin

   string bgtmp,image,bg1,bg2,namefile,bgsubmode
   int i,n

# Copy header parameters to local variables
   bgsubmode = bgtype

# The following three lines allow wildcards, @lists and comma-delimited image
# lists to be used - consistent with regular IRAF tasks
   namefile = mktemp ('tmp$tmp')
   files (images, sort+, > namefile)
   flist = namefile
   tlist = namefile

# advance tlist by 1 and check to see if list is empty
   if (fscan (tlist, image) == EOF) {
      delete (namefile, verify-)
      flist = ""
      tlist = ""
      print ("You didn't give me any images.  I'm really confused.  <whimper>")
      bye
      }

# check to see if .r images already exist
   i = strlen (image)
   if (substr (image, i-4, i) == '.fits')
   image = substr (image, 1, i-4)
   image = image//'.r.fits'
   if (access (image)) {
      delete (namefile, verify-)
      flist = ""
      tlist = ""
      print ("Well!  It looks like you've already got a "//image//"!")
      print ("Seems you don't need me after all... <sniffle>")
      bye
      }

# Make temporary file to hold averaged backgrounds
   bgtmp = mktemp ('tmp')

# median-combine input image list to make background if 'median' mode selected
   if (bgsubmode == 'median') {
      imcombine ('@'//namefile, bgtmp, nrejmasks = '', sigma = '',
         logfile = 'dev$null', combine = 'median', reject = 'none', project-,
         outtype = 'real', offsets = 'none', masktype = 'none', maskvalue = 0.,
         blank = 0., scale = 'none', zero = 'none', weight = 'none',
         statsec = '', expname = '', lthreshold = INDEF, hthreshold = INDEF,
         nlow = 0, nhigh = 7, nkeep = 1, mclip+, lsigma = 2, hsigma = 2,
         rdnoise = '', gain = '', snoise = '', sigscale = 0.1, pclip = -0.5,
         grow = 0)
      }

# main loop to scan through image list and subtract appropriate background
   n = 1
   while (fscan (flist, image) != EOF) {

# The following three lines strip the '.fits' if present
	 i = strlen (image)
	 if (substr (image, i-4, i) == '.fits')
	    image = substr (image, 1, i-5)

# normal and alternate subtraction modes - tlist should be one step ahead of
# flist and so gives you the next image on the list
      if (bgsubmode != 'median') {
         if (fscan (tlist, bg2) != EOF) {
            if (n == 1) imcopy (bg2, bgtmp, verbose-)
            else {
               imarith (bg1, '+', bg2, bgtmp, title = '',  divzero = 0.,
                  hparams = '', pixtype = 'real', calctype = 'real', verbose-,
                  noact-)
               imarith (bgtmp, '/', 2, bgtmp, title = '',  divzero = 0.,
                  hparams = '', pixtype = 'real', calctype = 'real', verbose-,
                  noact-)
               }
            }
         else imcopy (bg1, bgtmp, verbose-)
         n = n + 1
         bg1 = image
         }

# break out of the loop if we're in alternate mode and if we're on an odd n
# (which means we were on an even n before we did the n+1)
      if (bgsubmode == 'alt' && frac (real (n) / 2.) == 0.5) {
         imdelete (bgtmp, verify-)
         next
         }

# with median-subtraction mode, we don't have to do anything because we've
# already made the temporary background.  in any case, we've figured out what
# our background is at this point, so we'll subtract it.
      imarith (image, '-', bgtmp, image//'.r', title = '',  divzero = 0.,
         hparams = '', pixtype = 'real', calctype = 'real', verbose-, noact-)

# for non-median-subtraction mode, we need to delete the old bgtmp to make
# room for the next one
      if (bgsubmode != 'median') imdelete (bgtmp, verify-)

# add background-subtraction mode to image header
      hedit (image//'.r', 'bgsub', bgsubmode, add+, del-, ver-, show-, up+)

# print out background-subtracted image name
      print (image//'.r')

# close the while loop here
      }
         
# Clean up
   flist = ""
   tlist = ""
   delete (namefile, verify-)
   if (bgsubmode == 'median') imdelete (bgtmp, verify-)

end
